<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;

class RelRentabiProdutoPorEventoService {

	public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function findCaixa($CDFILIAL,$NRORG){
        $params = array (
            ':CDFILIAL' => $CDFILIAL,
            ':NRORG'    => $NRORG
 		);
 		
 		return $this->entityManager->getConnection()->fetchAll(self::GET_CAIXA, $params);
    }
    
    public function findEvento($conditions){
        $params = array (
            ':CDFILIAL'     => $conditions['CDFILIAL'],
            ':NRORG'        => $conditions['NRORG'],
            ':TCDFILIAL'    => $conditions['TCDFILIAL'],
            ':CDFILICAIXA'  => $conditions['CDFILICAIXA'],
            ':TCDCAIXA'     => $conditions['TCDCAIXA'],
            ':DTINI'        => $conditions['DTINI'],
            ':DTFIN'        => $conditions['DTFIN']
 		);
 		
 		$result = $this->entityManager->getConnection()->fetchAll(self::GET_EVENTO, $params);
 		
 		foreach($result as $key => $value){
 		    $result[$key]['DTINICIAL'] = explode(" ", $result[$key]['DTINICIAL']);
 		    $result[$key]['DTFINAL']   = explode(" ", $result[$key]['DTFINAL']);
 		    
            $result[$key]['DTINICIAL'] = $result[$key]['DTINICIAL'][0]." ". $result[$key]['HRINICIAL'];
            $result[$key]['DTFINAL']   = $result[$key]['DTFINAL'][0]." ". $result[$key]['HRFINAL'];
        }
        
        return $result;
    }
    

    public function find($conditions) {
        $result = [];
        
        $params = array(
         	':TIPOCUSTO'        => $conditions['CDTIPOCUSTO'],
         	':CDFILIAL'         => $conditions['CDFILIAL'],
         	':TCDFILIAL'        => $conditions['TCDFILIAL'],
         	':NRORG'            => $conditions['NRORG'],
         	':CDCAIXA'          => $conditions['CDCAIXA'],
            ':TCDCAIXA'         => $conditions['TCDCAIXA'],
         	':DTVENDAINI'       => $conditions['DTVENDAINI'],
         	':DTVENDAFIN'       => $conditions['DTVENDAFIN'],
         	':CDEVENTO'         => $conditions['CDEVENTO'],
         	':TCDEVENTO'        => $conditions['TCDEVENTO'],
         	':CDNVPRODUTOAGRUP' => $conditions['CDNVPRODUTOAGRUP'],
         	':PRODNIV'          => $conditions['PRODNIV'],
         	':TCDPRODUTO'       => $conditions['TCDPRODUTO'],
         	':CDPRODLIST'       => $conditions['CDPRODLIST'],
         	':CDARVPRODF'       => $conditions['CDARVPRODF'],
         	':CDARVPRODI'       => $conditions['CDARVPRODI']
 		);
 		
 		if ($conditions['FLAGIMPOSTO'] == 'C') {
 		    $result = $this->entityManager->getConnection()->fetchAll(self::GET_RENTABILIDADE_PRODUTO, $params);
 		} else {
 		    $result = $this->entityManager->getConnection()->fetchAll(self::GET_RENTABILIDADE_PRODUTO_SEM_IMPOSTO, $params);
 		}
 		
 		foreach($result as $chTR => $valTR){
 	        $result[$chTR]['LABELEVENTO'] = $result[$chTR]['CDEVENTO']." - ". $result[$chTR]['NMEVENTO'];
 	        $result[$chTR]['LABELLOJACAIXA'] = $result[$chTR]['CDLOJA']." - ". $result[$chTR]['NMLOJA']." | ".$result[$chTR]['CDCAIXA']." - ". $result[$chTR]['NMCAIXA'];
        }
            
 		return $result;
    }


    const GET_CAIXA = "
        SELECT CDCAIXA, NMCAIXA
            FROM CAIXAS C
            WHERE C.CDFILIAL = :CDFILIAL
    ";

    
    const GET_EVENTO = "
        SELECT EC.CDEVENTO, EV.NMEVENTO, 
                EV.DTINICIAL, EV.DTFINAL,
                EV.HRINICIAL, EV.HRFINAL,
                EF.CDFILIAL || ' - ' || F.NMFILIAL AS LABELFILIAL, 
                EC.CDCAIXA || ' - ' || C.NMCAIXA AS LABELCAIXA  
            FROM EVENTOCAIXA EC
                JOIN EVENTOFILIAL EF 
                    ON EC.CDEVENTO = EF.CDEVENTO
                        AND EC.CDFILIAL = EF.CDFILIAL
                        AND EC.NRORG = EF.NRORG
                JOIN EVENTOVENDA EV 
                    ON EC.CDEVENTO = EV.CDEVENTO
                        AND EC.NRORG = EV.NRORG
                JOIN FILIAL F
                    ON F.CDFILIAL = EF.CDFILIAL
                        AND F.CDFILIAL = EC.CDFILIAL
                LEFT JOIN CAIXA C
                    ON C.CDCAIXA = EC.CDCAIXA
                        AND C.CDFILIAL = EC.CDFILIAL 
                        AND C.CDFILIAL = EF.CDFILIAL
                        AND C.CDFILIAL = F.CDFILIAL
            WHERE ((:CDFILIAL LIKE '%' || EF.CDFILIAL || '%') OR (:TCDFILIAL = 'T'))
                AND EF.NRORG = :NRORG
                AND ((EV.DTINICIAL BETWEEN :DTINI AND :DTFIN) OR (EV.DTFINAL BETWEEN :DTINI AND :DTFIN))
                AND ((INSTR(:CDFILICAIXA , EC.CDFILIAL||EC.CDCAIXA) <> 0) OR (:TCDCAIXA = 'T'))
    ";
    
    const GET_RENTABILIDADE_PRODUTO = "
        SELECT CDEVENTO, NMEVENTO, CDFILIAL||' - '||NMFILIAL AS LABELFILIAL,
                CDLOJA||' - '||NMLOJA AS LABELLOJA, CDFILIAL, NMFILIAL, CDLOJA, 
                NMLOJA, CDCAIXA, NMCAIXA, CDPRODUTO, CDARVPROD, NMPRODUTO, VRUNITVEND, CDPRODUTOAVO, NMPRODUTOAVO,
                SUM(QTPRODVEND) AS QTPRODVEND, SUM(VRIMPOSTO) AS VRIMPOSTO,
                SUM(VRTOTAL) AS VRTOTAL, SUM(VRCUSTO) AS VRCUSTO,
                SUM(VRTOTAL) - SUM(VRIMPOSTO) - SUM(VRCUSTO) AS VRRENTA,
                CDTIPOCUSTO, NMTIPOCUSTO, IDCUSFOPRECO,
                ROUND((SUM(VRCUSTO)/DECODE(SUM(NVL(QTPRODVEND, 1)), 0, 1, SUM(NVL(QTPRODVEND, 1)))), 2) AS CUSTOUNIT,
                ROUND(((ROUND((SUM(VRCUSTO)/DECODE(SUM(NVL(QTPRODVEND, 1)), 0, 1, SUM(NVL(QTPRODVEND, 1)))), 2)/DECODE(VRUNITVEND, 0, 1, VRUNITVEND)) * 100) , 2) AS PERCCUS,
                ROUND((((ROUND((SUM(VRCUSTO)/DECODE(SUM(NVL(QTPRODVEND, 1)), 0, 1, SUM(NVL(QTPRODVEND, 1)))), 2)/DECODE(VRUNITVEND, 0, 1, VRUNITVEND)) * 100)*-1), 2) + 100 AS PERCLUCR
            FROM (
                SELECT FILOCAEV.CDFILIAL, FILOCAEV.NMFILIAL, FILOCAEV.CDEVENTO, FILOCAEV.NMEVENTO, FILOCAEV.CDLOJA, 
                        FILOCAEV.NMLOJA, FILOCAEV.CDCAIXA, FILOCAEV.NMCAIXA, I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, 
                        CASE I.VRUNITVEND 
                            WHEN 0 
                                THEN (I.VRUNITVEND + NVL(I.VRACRITVEND,0)) 
                            ELSE I.VRUNITVEND 
                        END AS VRUNITVEND, 
                        SUM(I.QTPRODVEND) AS QTPRODVEND, 
                        SUM((TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) - NVL(I.VRDESITVEND,0) + NVL(I.VRACRITVEND,0)) * M.VRPEALPRODIT / 100) AS VRIMPOSTO,
                        SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) - NVL(I.VRDESITVEND,0) + NVL(I.VRACRITVEND,0)) AS VRTOTAL,
                        SUM(CUSTO_COMPOSICAO(I.CDPRODUTO, FILOCAEV.CDFILIAL, FILOCAEV.CDFILIAL, :TIPOCUSTO, I.QTPRODVEND, FILOCAEV.CDLOJA, FILOCAEV.DTENTRVENDA)) AS VRCUSTO,
                        :TIPOCUSTO AS CDTIPOCUSTO, :TIPOCUSTO AS NMTIPOCUSTO, :TIPOCUSTO AS IDCUSFOPRECO, PV.CDPRODUTO AS CDPRODUTOAVO, PV.NMPRODUTO AS NMPRODUTOAVO
                    FROM ITEMVENDA I
                        JOIN PRODUTO P
                            ON I.CDPRODUTO = P.CDPRODUTO
                                AND I.NRORG = P.NRORG
                        JOIN PRODUTO PV
                            ON P.NRORG = PV.NRORG
                        JOIN ITVENDAIMPOS M
                            ON I.CDFILIAL = M.CDFILIAL
                                AND I.CDCAIXA = M.CDCAIXA
                                AND I.NRSEQVENDA = M.NRSEQVENDA
                                AND I.NRSEQUITVEND = M.NRSEQUITVEND
                        JOIN (
                            SELECT FILOCAEV.CDEVENTO, FILOCAEV.NMEVENTO, V.IDSITUVENDA, V.NRSEQVENDA, FILOCAEV.CDCAIXA,
                                    FILOCAEV.NMCAIXA, FILOCAEV.CDLOJA, FILOCAEV.NMLOJA, FILOCAEV.CDFILIAL, 
                                    FILOCAEV.NMFILIAL, FILOCAEV.NRORG, FILOCAEV.CDPRODTAXASERV, FILOCAEV.CDPRODTAXAENTR, V.DTENTRVENDA
                                FROM VENDA V
                                    JOIN (
                                        SELECT EVENTOS.CDFILIAL, EVENTOS.CDEVENTO, EVENTOS.NMEVENTO, EVENTOS.CDCAIXA, 
                                                EVENTOS.DSEVENTO, EVENTOS.DTINICIAL, EVENTOS.DTFINAL,
                                                FILOCA.NMCAIXA, FILOCA.CDLOJA, FILOCA.NMLOJA, FILOCA.NMFILIAL, 
                                                FILOCA.NRORG, FILOCA.CDPRODTAXASERV, FILOCA.CDPRODTAXAENTR
                                            FROM (
                                                SELECT EF.CDFILIAL, EC.CDEVENTO, EV.NMEVENTO, EC.CDCAIXA, EV.DSEVENTO, EV.DTINICIAL, EV.DTFINAL
                                                    FROM EVENTOCAIXA EC
                                                        JOIN EVENTOFILIAL EF 
                                                            ON EC.CDEVENTO = EF.CDEVENTO
                                                                AND EC.CDFILIAL = EF.CDFILIAL
                                                                AND EC.NRORG = EF.NRORG
                                                        JOIN EVENTOVENDA EV 
                                                            ON EC.CDEVENTO = EV.CDEVENTO
                                                                AND EC.NRORG = EV.NRORG
                                                    WHERE ((INSTR(:CDFILIAL , EF.CDFILIAL ) <> 0) OR (:TCDFILIAL = 'T'))
                                                        AND EF.NRORG = :NRORG
                                                        AND (((EV.CDEVENTO = :CDEVENTO)) OR (:TCDEVENTO = 'T'))
                                            ) EVENTOS
                                            JOIN ( 
                                                SELECT C.CDCAIXA, C.NMCAIXA, FILO.CDLOJA, FILO.NMLOJA, FILO.CDFILIAL, 
                                                        FILO.NMFILIAL, FILO.NRORG, FILO.CDPRODTAXASERV, FILO.CDPRODTAXAENTR
                                                    FROM CAIXA C
                                                    JOIN (
                                                        SELECT L.CDLOJA, L.NMLOJA, FI.CDFILIAL, FI.NMFILIAL, FI.NRORG,
                                                                L.CDPRODTAXASERV, L.CDPRODTAXAENTR
                                                            FROM LOJA L
                                                            JOIN ( 
                                                                SELECT F.CDFILIAL, F.NMFILIAL, F.NRORG
                                                                    FROM FILIAL F
                                                                    WHERE ((INSTR(:CDFILIAL, F.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
                                                                        AND F.NRORG = :NRORG 
                                                            ) FI
                                                                ON L.CDFILIAL = FI.CDFILIAL
                                                                    AND L.NRORG = FI.NRORG 
                                                    ) FILO
                                                        ON C.CDFILIAL = FILO.CDFILIAL
                                                            AND C.CDLOJA = FILO.CDLOJA
                                                            AND C.NRORG = FILO.NRORG 
                                                    WHERE ((INSTR(:CDCAIXA, C.CDCAIXA) <> 0) OR (:TCDCAIXA = 'T'))
                                            ) FILOCA
                                        ON EVENTOS.CDCAIXA = FILOCA.CDCAIXA 
                                    ) FILOCAEV
                                        ON V.CDCAIXA = FILOCAEV.CDCAIXA
                                            AND V.CDLOJA = FILOCAEV.CDLOJA
                                            AND V.CDFILIAL = FILOCAEV.CDFILIAL
                                            AND V.NRORG = FILOCAEV.NRORG
                                WHERE V.IDSITUVENDA = 'O'
                                    AND V.DTENTRVENDA BETWEEN :DTVENDAINI AND :DTVENDAFIN
                                    AND V.DTENTRVENDA BETWEEN FILOCAEV.DTINICIAL AND FILOCAEV.DTFINAL 
                        ) FILOCAEV
                            ON FILOCAEV.CDFILIAL = M.CDFILIAL
                                AND FILOCAEV.CDCAIXA = M.CDCAIXA
                                AND FILOCAEV.NRSEQVENDA = M.NRSEQVENDA
                    WHERE (((((INSTR(:CDPRODLIST, P.CDPRODUTO) > 0 ) OR (P.CDARVPROD BETWEEN :CDARVPRODI AND :CDARVPRODF ))) OR (P.CDARVPROD LIKE :PRODNIV)) OR (:TCDPRODUTO = 'T'))
                        AND P.CDARVPROD LIKE PV.CDARVPROD||'%'
                        AND PV.CDNVPRODUTO = :CDNVPRODUTOAGRUP
                        AND PV.NRORG = :NRORG
                    GROUP BY FILOCAEV.CDFILIAL, FILOCAEV.NMFILIAL, FILOCAEV.CDEVENTO, FILOCAEV.NMEVENTO, 
                            FILOCAEV.CDLOJA, FILOCAEV.NMLOJA, FILOCAEV.CDCAIXA, FILOCAEV.NMCAIXA, I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, 
                            I.VRUNITVEND, I.VRACRITVEND, :TIPOCUSTO, PV.CDPRODUTO, PV.NMPRODUTO
                    
                UNION
                    
                    SELECT FILOCAEV.CDFILIAL, FILOCAEV.NMFILIAL, FILOCAEV.CDEVENTO, FILOCAEV.NMEVENTO, 
                            FILOCAEV.CDLOJA, FILOCAEV.NMLOJA, FILOCAEV.CDCAIXA, FILOCAEV.NMCAIXA, I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO,  
                            CASE I.VRUNITVEND 
                                WHEN 0 
                                    THEN (I.VRUNITVEND + NVL(I.VRACRITVEND,0)) 
                                ELSE I.VRUNITVEND 
                            END AS VRUNITVEND, 
                            0 AS QTPRODVEND, 0 AS VRIMPOSTO, 0 AS VRTOTAL,
                            SUM(CUSTO_COMPOSICAO(M.CDPRODUTO, FILOCAEV.CDFILIAL, FILOCAEV.CDFILIAL, :TIPOCUSTO, M.QTITVENDAEST, FILOCAEV.CDLOJA, FILOCAEV.DTENTRVENDA)) AS VRCUSTO,
                            :TIPOCUSTO AS CDTIPOCUSTO, :TIPOCUSTO AS NMTIPOCUSTO, :TIPOCUSTO AS IDCUSFOPRECO, 
                            PV.CDPRODUTO AS CDPRODUTOAVO, PV.NMPRODUTO AS NMPRODUTOAVO
                        FROM ITEMVENDA I
                            JOIN PRODUTO P
                                ON I.CDPRODUTO = P.CDPRODUTO
                                    AND I.NRORG = P.NRORG
                            JOIN PRODUTO PV
                                ON P.NRORG = PV.NRORG
                            JOIN ITEMVENDAEST M
                                ON I.CDFILIAL = M.CDFILIAL
                                    AND I.CDCAIXA = M.CDCAIXA
                                    AND I.NRSEQVENDA = M.NRSEQVENDA
                                    AND I.NRSEQUITVEND = M.NRSEQUITVEND
                            JOIN (
                                SELECT FILOCAEV.CDEVENTO, FILOCAEV.NMEVENTO, V.IDSITUVENDA, V.NRSEQVENDA, FILOCAEV.CDCAIXA, 
                                        FILOCAEV.NMCAIXA, FILOCAEV.CDLOJA, FILOCAEV.NMLOJA, FILOCAEV.CDFILIAL, 
                                        FILOCAEV.NMFILIAL, FILOCAEV.NRORG, FILOCAEV.CDPRODTAXASERV, FILOCAEV.CDPRODTAXAENTR, V.DTENTRVENDA
                                    FROM VENDA V
                                        JOIN (
                                            SELECT EVENTOS.CDFILIAL, EVENTOS.CDEVENTO, EVENTOS.NMEVENTO, EVENTOS.CDCAIXA, 
                                                    EVENTOS.DSEVENTO, EVENTOS.DTINICIAL, EVENTOS.DTFINAL,
                                                    FILOCA.NMCAIXA, FILOCA.CDLOJA, FILOCA.NMLOJA, FILOCA.NMFILIAL, 
                                                    FILOCA.NRORG, FILOCA.CDPRODTAXASERV, FILOCA.CDPRODTAXAENTR
                                                FROM (
                                                    SELECT EF.CDFILIAL, EC.CDEVENTO, EV.NMEVENTO, EC.CDCAIXA, EV.DSEVENTO, EV.DTINICIAL, EV.DTFINAL
                                                        FROM EVENTOCAIXA EC
                                                            JOIN EVENTOFILIAL EF 
                                                                ON EC.CDEVENTO = EF.CDEVENTO
                                                                    AND EC.CDFILIAL = EF.CDFILIAL
                                                                    AND EC.NRORG = EF.NRORG
                                                            JOIN EVENTOVENDA EV 
                                                                ON EC.CDEVENTO = EV.CDEVENTO
                                                                    AND EC.NRORG = EV.NRORG
                                                        WHERE ((INSTR(:CDFILIAL , EF.CDFILIAL ) <> 0) OR (:TCDFILIAL = 'T'))
                                                            AND EF.NRORG = :NRORG
                                                            AND (((EV.CDEVENTO = :CDEVENTO)) OR (:TCDEVENTO = 'T'))
                                                ) EVENTOS
                                                    JOIN (
                                                        SELECT C.CDCAIXA, C.NMCAIXA, FILO.CDLOJA, FILO.NMLOJA, FILO.CDFILIAL, FILO.NMFILIAL, 
                                                                FILO.NRORG, FILO.CDPRODTAXASERV, FILO.CDPRODTAXAENTR
                                                            FROM CAIXA C
                                                                JOIN (
                                                                    SELECT L.CDLOJA, L.NMLOJA, FI.CDFILIAL, FI.NMFILIAL, FI.NRORG, L.CDPRODTAXASERV, L.CDPRODTAXAENTR
                                                                        FROM LOJA L
                                                                            JOIN (
                                                                                SELECT F.CDFILIAL, F.NMFILIAL, F.NRORG
                                                                                    FROM FILIAL F
                                                                                    WHERE ((INSTR(:CDFILIAL , F.CDFILIAL ) <> 0) OR (:TCDFILIAL = 'T'))
                                                                                        AND F.NRORG = :NRORG 
                                                                            ) FI
                                                                                ON L.CDFILIAL = FI.CDFILIAL
                                                                                    AND L.NRORG = FI.NRORG 
                                                                ) FILO
                                                                    ON C.CDFILIAL = FILO.CDFILIAL
                                                                        AND C.CDLOJA = FILO.CDLOJA
                                                                        AND C.NRORG = FILO.NRORG 
                                                            WHERE ((INSTR(:CDCAIXA, C.CDCAIXA) <> 0) OR (:TCDCAIXA = 'T'))
                                                    ) FILOCA
                                                        ON EVENTOS.CDCAIXA = FILOCA.CDCAIXA 
                                        ) FILOCAEV
                                            ON V.CDCAIXA = FILOCAEV.CDCAIXA
                                                AND V.CDLOJA = FILOCAEV.CDLOJA
                                                AND V.CDFILIAL = FILOCAEV.CDFILIAL
                                                AND V.NRORG = FILOCAEV.NRORG
                                    WHERE V.IDSITUVENDA = 'O'
                                        AND V.DTENTRVENDA BETWEEN :DTVENDAINI AND :DTVENDAFIN
                                        AND V.DTENTRVENDA BETWEEN FILOCAEV.DTINICIAL AND FILOCAEV.DTFINAL 
                            ) FILOCAEV
                                ON FILOCAEV.CDFILIAL = M.CDFILIAL
                                    AND FILOCAEV.CDCAIXA = M.CDCAIXA
                                    AND FILOCAEV.NRSEQVENDA = M.NRSEQVENDA
                        WHERE (((((INSTR(:CDPRODLIST, P.CDPRODUTO) > 0) OR (P.CDARVPROD BETWEEN :CDARVPRODI AND :CDARVPRODF))) OR (P.CDARVPROD LIKE :PRODNIV)) OR (:TCDPRODUTO = 'T'))
                            AND P.CDARVPROD LIKE PV.CDARVPROD||'%'
                            AND PV.CDNVPRODUTO = :CDNVPRODUTOAGRUP
                            AND PV.NRORG = :NRORG
                        GROUP BY FILOCAEV.CDFILIAL, FILOCAEV.NMFILIAL, FILOCAEV.CDEVENTO, FILOCAEV.NMEVENTO, 
                                FILOCAEV.CDLOJA, FILOCAEV.NMLOJA, FILOCAEV.CDCAIXA, FILOCAEV.NMCAIXA, I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, 
                                I.VRUNITVEND, I.VRACRITVEND, :TIPOCUSTO, PV.CDPRODUTO, PV.NMPRODUTO
            )
            GROUP BY CDFILIAL, NMFILIAL, CDEVENTO, NMEVENTO, CDLOJA, NMLOJA, CDCAIXA, NMCAIXA, CDPRODUTO, CDARVPROD, 
                    NMPRODUTO, VRUNITVEND, CDTIPOCUSTO, NMTIPOCUSTO, IDCUSFOPRECO, CDPRODUTOAVO, NMPRODUTOAVO
        ORDER BY CDFILIAL ASC, CDLOJA ASC, VRRENTA DESC
    ";
    
    const GET_RENTABILIDADE_PRODUTO_SEM_IMPOSTO = "
        SELECT CDEVENTO, NMEVENTO, CDFILIAL||' - '||NMFILIAL AS LABELFILIAL,
                CDLOJA||' - '||NMLOJA AS LABELLOJA, CDFILIAL, NMFILIAL, CDLOJA, 
                NMLOJA, CDCAIXA, NMCAIXA, CDPRODUTO, CDARVPROD, NMPRODUTO, VRUNITVEND, CDPRODUTOAVO, NMPRODUTOAVO,
                SUM(QTPRODVEND) AS QTPRODVEND, SUM(VRIMPOSTO) AS VRIMPOSTO,
                SUM(VRTOTAL) AS VRTOTAL, SUM(VRCUSTO) AS VRCUSTO,
                SUM(VRTOTAL) - SUM(VRIMPOSTO) - SUM(VRCUSTO) AS VRRENTA,
                CDTIPOCUSTO, NMTIPOCUSTO, IDCUSFOPRECO,
                ROUND((SUM(VRCUSTO)/DECODE(SUM(NVL(QTPRODVEND, 1)), 0, 1, SUM(NVL(QTPRODVEND, 1)))), 2) AS CUSTOUNIT,
                ROUND(((ROUND((SUM(VRCUSTO)/DECODE(SUM(NVL(QTPRODVEND, 1)), 0, 1, SUM(NVL(QTPRODVEND, 1)))), 2)/DECODE(VRUNITVEND, 0, 1, VRUNITVEND)) * 100) , 2) AS PERCCUS,
                ROUND((((ROUND((SUM(VRCUSTO)/DECODE(SUM(NVL(QTPRODVEND, 1)), 0, 1, SUM(NVL(QTPRODVEND, 1)))), 2)/DECODE(VRUNITVEND, 0, 1, VRUNITVEND)) * 100)*-1), 2) + 100 AS PERCLUCR
            FROM (
                SELECT FILOCAEV.CDFILIAL, FILOCAEV.NMFILIAL, FILOCAEV.CDEVENTO, FILOCAEV.NMEVENTO, FILOCAEV.CDLOJA, 
                        FILOCAEV.NMLOJA, FILOCAEV.CDCAIXA, FILOCAEV.NMCAIXA, I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, 
                        CASE I.VRUNITVEND 
                            WHEN 0 
                                THEN (I.VRUNITVEND + NVL(I.VRACRITVEND,0)) 
                            ELSE I.VRUNITVEND 
                        END AS VRUNITVEND, 
                        SUM(I.QTPRODVEND) AS QTPRODVEND, 
                        0 AS VRIMPOSTO,
                        SUM(TRUNC(I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL,0)),2) - NVL(I.VRDESITVEND,0) + NVL(I.VRACRITVEND,0)) AS VRTOTAL,
                        SUM(CUSTO_COMPOSICAO(I.CDPRODUTO, FILOCAEV.CDFILIAL, FILOCAEV.CDFILIAL, :TIPOCUSTO, I.QTPRODVEND, FILOCAEV.CDLOJA, FILOCAEV.DTENTRVENDA)) AS VRCUSTO,
                        :TIPOCUSTO AS CDTIPOCUSTO, :TIPOCUSTO AS NMTIPOCUSTO, :TIPOCUSTO AS IDCUSFOPRECO, PV.CDPRODUTO AS CDPRODUTOAVO, PV.NMPRODUTO AS NMPRODUTOAVO
                    FROM ITEMVENDA I
                        JOIN PRODUTO P
                            ON I.CDPRODUTO = P.CDPRODUTO
                                AND I.NRORG = P.NRORG
                        JOIN PRODUTO PV
                            ON P.NRORG = PV.NRORG
                        JOIN ITVENDAIMPOS M
                            ON I.CDFILIAL = M.CDFILIAL
                                AND I.CDCAIXA = M.CDCAIXA
                                AND I.NRSEQVENDA = M.NRSEQVENDA
                                AND I.NRSEQUITVEND = M.NRSEQUITVEND
                        JOIN (
                            SELECT FILOCAEV.CDEVENTO, FILOCAEV.NMEVENTO, V.IDSITUVENDA, V.NRSEQVENDA, FILOCAEV.CDCAIXA,
                                    FILOCAEV.NMCAIXA, FILOCAEV.CDLOJA, FILOCAEV.NMLOJA, FILOCAEV.CDFILIAL, 
                                    FILOCAEV.NMFILIAL, FILOCAEV.NRORG, FILOCAEV.CDPRODTAXASERV, FILOCAEV.CDPRODTAXAENTR, V.DTENTRVENDA
                                FROM VENDA V
                                    JOIN (
                                        SELECT EVENTOS.CDFILIAL, EVENTOS.CDEVENTO, EVENTOS.NMEVENTO, EVENTOS.CDCAIXA, 
                                                EVENTOS.DSEVENTO, EVENTOS.DTINICIAL, EVENTOS.DTFINAL,
                                                FILOCA.NMCAIXA, FILOCA.CDLOJA, FILOCA.NMLOJA, FILOCA.NMFILIAL, 
                                                FILOCA.NRORG, FILOCA.CDPRODTAXASERV, FILOCA.CDPRODTAXAENTR
                                            FROM (
                                                SELECT EF.CDFILIAL, EC.CDEVENTO, EV.NMEVENTO, EC.CDCAIXA, EV.DSEVENTO, EV.DTINICIAL, EV.DTFINAL
                                                    FROM EVENTOCAIXA EC
                                                        JOIN EVENTOFILIAL EF 
                                                            ON EC.CDEVENTO = EF.CDEVENTO
                                                                AND EC.CDFILIAL = EF.CDFILIAL
                                                                AND EC.NRORG = EF.NRORG
                                                        JOIN EVENTOVENDA EV 
                                                            ON EC.CDEVENTO = EV.CDEVENTO
                                                                AND EC.NRORG = EV.NRORG
                                                    WHERE ((INSTR(:CDFILIAL , EF.CDFILIAL ) <> 0) OR (:TCDFILIAL = 'T'))
                                                        AND EF.NRORG = :NRORG
                                                        AND (((EV.CDEVENTO = :CDEVENTO)) OR (:TCDEVENTO = 'T'))
                                            ) EVENTOS
                                            JOIN ( 
                                                SELECT C.CDCAIXA, C.NMCAIXA, FILO.CDLOJA, FILO.NMLOJA, FILO.CDFILIAL, 
                                                        FILO.NMFILIAL, FILO.NRORG, FILO.CDPRODTAXASERV, FILO.CDPRODTAXAENTR
                                                    FROM CAIXA C
                                                    JOIN (
                                                        SELECT L.CDLOJA, L.NMLOJA, FI.CDFILIAL, FI.NMFILIAL, FI.NRORG,
                                                                L.CDPRODTAXASERV, L.CDPRODTAXAENTR
                                                            FROM LOJA L
                                                            JOIN ( 
                                                                SELECT F.CDFILIAL, F.NMFILIAL, F.NRORG
                                                                    FROM FILIAL F
                                                                    WHERE ((INSTR(:CDFILIAL, F.CDFILIAL) <> 0) OR (:TCDFILIAL = 'T'))
                                                                        AND F.NRORG = :NRORG 
                                                            ) FI
                                                                ON L.CDFILIAL = FI.CDFILIAL
                                                                    AND L.NRORG = FI.NRORG 
                                                    ) FILO
                                                        ON C.CDFILIAL = FILO.CDFILIAL
                                                            AND C.CDLOJA = FILO.CDLOJA
                                                            AND C.NRORG = FILO.NRORG 
                                                    WHERE ((INSTR(:CDCAIXA, C.CDCAIXA) <> 0) OR (:TCDCAIXA = 'T'))
                                            ) FILOCA
                                        ON EVENTOS.CDCAIXA = FILOCA.CDCAIXA 
                                    ) FILOCAEV
                                        ON V.CDCAIXA = FILOCAEV.CDCAIXA
                                            AND V.CDLOJA = FILOCAEV.CDLOJA
                                            AND V.CDFILIAL = FILOCAEV.CDFILIAL
                                            AND V.NRORG = FILOCAEV.NRORG
                                WHERE V.IDSITUVENDA = 'O'
                                    AND V.DTENTRVENDA BETWEEN :DTVENDAINI AND :DTVENDAFIN
                                    AND V.DTENTRVENDA BETWEEN FILOCAEV.DTINICIAL AND FILOCAEV.DTFINAL 
                        ) FILOCAEV
                            ON FILOCAEV.CDFILIAL = M.CDFILIAL
                                AND FILOCAEV.CDCAIXA = M.CDCAIXA
                                AND FILOCAEV.NRSEQVENDA = M.NRSEQVENDA
                    WHERE (((((INSTR(:CDPRODLIST, P.CDPRODUTO) > 0 ) OR (P.CDARVPROD BETWEEN :CDARVPRODI AND :CDARVPRODF ))) OR (P.CDARVPROD LIKE :PRODNIV)) OR (:TCDPRODUTO = 'T'))
                        AND P.CDARVPROD LIKE PV.CDARVPROD||'%'
                        AND PV.CDNVPRODUTO = :CDNVPRODUTOAGRUP
                        AND PV.NRORG = :NRORG
                    GROUP BY FILOCAEV.CDFILIAL, FILOCAEV.NMFILIAL, FILOCAEV.CDEVENTO, FILOCAEV.NMEVENTO, 
                            FILOCAEV.CDLOJA, FILOCAEV.NMLOJA, FILOCAEV.CDCAIXA, FILOCAEV.NMCAIXA, I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, 
                            I.VRUNITVEND, I.VRACRITVEND, :TIPOCUSTO, PV.CDPRODUTO, PV.NMPRODUTO
                    
                UNION
                    
                    SELECT FILOCAEV.CDFILIAL, FILOCAEV.NMFILIAL, FILOCAEV.CDEVENTO, FILOCAEV.NMEVENTO, 
                            FILOCAEV.CDLOJA, FILOCAEV.NMLOJA, FILOCAEV.CDCAIXA, FILOCAEV.NMCAIXA, I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO,  
                            CASE I.VRUNITVEND 
                                WHEN 0 
                                    THEN (I.VRUNITVEND + NVL(I.VRACRITVEND,0)) 
                                ELSE I.VRUNITVEND 
                            END AS VRUNITVEND, 
                            0 AS QTPRODVEND, 0 AS VRIMPOSTO, 0 AS VRTOTAL,
                            SUM(CUSTO_COMPOSICAO(M.CDPRODUTO, FILOCAEV.CDFILIAL, FILOCAEV.CDFILIAL, :TIPOCUSTO, M.QTITVENDAEST, FILOCAEV.CDLOJA, FILOCAEV.DTENTRVENDA)) AS VRCUSTO,
                            :TIPOCUSTO AS CDTIPOCUSTO, :TIPOCUSTO AS NMTIPOCUSTO, :TIPOCUSTO AS IDCUSFOPRECO, 
                            PV.CDPRODUTO AS CDPRODUTOAVO, PV.NMPRODUTO AS NMPRODUTOAVO
                        FROM ITEMVENDA I
                            JOIN PRODUTO P
                                ON I.CDPRODUTO = P.CDPRODUTO
                                    AND I.NRORG = P.NRORG
                            JOIN PRODUTO PV
                                ON P.NRORG = PV.NRORG
                            JOIN ITEMVENDAEST M
                                ON I.CDFILIAL = M.CDFILIAL
                                    AND I.CDCAIXA = M.CDCAIXA
                                    AND I.NRSEQVENDA = M.NRSEQVENDA
                                    AND I.NRSEQUITVEND = M.NRSEQUITVEND
                            JOIN (
                                SELECT FILOCAEV.CDEVENTO, FILOCAEV.NMEVENTO, V.IDSITUVENDA, V.NRSEQVENDA, FILOCAEV.CDCAIXA, 
                                        FILOCAEV.NMCAIXA, FILOCAEV.CDLOJA, FILOCAEV.NMLOJA, FILOCAEV.CDFILIAL, 
                                        FILOCAEV.NMFILIAL, FILOCAEV.NRORG, FILOCAEV.CDPRODTAXASERV, FILOCAEV.CDPRODTAXAENTR, V.DTENTRVENDA
                                    FROM VENDA V
                                        JOIN (
                                            SELECT EVENTOS.CDFILIAL, EVENTOS.CDEVENTO, EVENTOS.NMEVENTO, EVENTOS.CDCAIXA, 
                                                    EVENTOS.DSEVENTO, EVENTOS.DTINICIAL, EVENTOS.DTFINAL,
                                                    FILOCA.NMCAIXA, FILOCA.CDLOJA, FILOCA.NMLOJA, FILOCA.NMFILIAL, 
                                                    FILOCA.NRORG, FILOCA.CDPRODTAXASERV, FILOCA.CDPRODTAXAENTR
                                                FROM (
                                                    SELECT EF.CDFILIAL, EC.CDEVENTO, EV.NMEVENTO, EC.CDCAIXA, EV.DSEVENTO, EV.DTINICIAL, EV.DTFINAL
                                                        FROM EVENTOCAIXA EC
                                                            JOIN EVENTOFILIAL EF 
                                                                ON EC.CDEVENTO = EF.CDEVENTO
                                                                    AND EC.CDFILIAL = EF.CDFILIAL
                                                                    AND EC.NRORG = EF.NRORG
                                                            JOIN EVENTOVENDA EV 
                                                                ON EC.CDEVENTO = EV.CDEVENTO
                                                                    AND EC.NRORG = EV.NRORG
                                                        WHERE ((INSTR(:CDFILIAL , EF.CDFILIAL ) <> 0) OR (:TCDFILIAL = 'T'))
                                                            AND EF.NRORG = :NRORG
                                                            AND (((EV.CDEVENTO = :CDEVENTO)) OR (:TCDEVENTO = 'T'))
                                                ) EVENTOS
                                                    JOIN (
                                                        SELECT C.CDCAIXA, C.NMCAIXA, FILO.CDLOJA, FILO.NMLOJA, FILO.CDFILIAL, FILO.NMFILIAL, 
                                                                FILO.NRORG, FILO.CDPRODTAXASERV, FILO.CDPRODTAXAENTR
                                                            FROM CAIXA C
                                                                JOIN (
                                                                    SELECT L.CDLOJA, L.NMLOJA, FI.CDFILIAL, FI.NMFILIAL, FI.NRORG, L.CDPRODTAXASERV, L.CDPRODTAXAENTR
                                                                        FROM LOJA L
                                                                            JOIN (
                                                                                SELECT F.CDFILIAL, F.NMFILIAL, F.NRORG
                                                                                    FROM FILIAL F
                                                                                    WHERE ((INSTR(:CDFILIAL , F.CDFILIAL ) <> 0) OR (:TCDFILIAL = 'T'))
                                                                                        AND F.NRORG = :NRORG 
                                                                            ) FI
                                                                                ON L.CDFILIAL = FI.CDFILIAL
                                                                                    AND L.NRORG = FI.NRORG 
                                                                ) FILO
                                                                    ON C.CDFILIAL = FILO.CDFILIAL
                                                                        AND C.CDLOJA = FILO.CDLOJA
                                                                        AND C.NRORG = FILO.NRORG 
                                                            WHERE ((INSTR(:CDCAIXA, C.CDCAIXA) <> 0) OR (:TCDCAIXA = 'T'))
                                                    ) FILOCA
                                                        ON EVENTOS.CDCAIXA = FILOCA.CDCAIXA 
                                        ) FILOCAEV
                                            ON V.CDCAIXA = FILOCAEV.CDCAIXA
                                                AND V.CDLOJA = FILOCAEV.CDLOJA
                                                AND V.CDFILIAL = FILOCAEV.CDFILIAL
                                                AND V.NRORG = FILOCAEV.NRORG
                                    WHERE V.IDSITUVENDA = 'O'
                                        AND V.DTENTRVENDA BETWEEN :DTVENDAINI AND :DTVENDAFIN
                                        AND V.DTENTRVENDA BETWEEN FILOCAEV.DTINICIAL AND FILOCAEV.DTFINAL 
                            ) FILOCAEV
                                ON FILOCAEV.CDFILIAL = M.CDFILIAL
                                    AND FILOCAEV.CDCAIXA = M.CDCAIXA
                                    AND FILOCAEV.NRSEQVENDA = M.NRSEQVENDA
                        WHERE (((((INSTR(:CDPRODLIST, P.CDPRODUTO) > 0) OR (P.CDARVPROD BETWEEN :CDARVPRODI AND :CDARVPRODF))) OR (P.CDARVPROD LIKE :PRODNIV)) OR (:TCDPRODUTO = 'T'))
                            AND P.CDARVPROD LIKE PV.CDARVPROD||'%'
                            AND PV.CDNVPRODUTO = :CDNVPRODUTOAGRUP
                            AND PV.NRORG = :NRORG
                        GROUP BY FILOCAEV.CDFILIAL, FILOCAEV.NMFILIAL, FILOCAEV.CDEVENTO, FILOCAEV.NMEVENTO, 
                                FILOCAEV.CDLOJA, FILOCAEV.NMLOJA, FILOCAEV.CDCAIXA, FILOCAEV.NMCAIXA, I.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, 
                                I.VRUNITVEND, I.VRACRITVEND, :TIPOCUSTO, PV.CDPRODUTO, PV.NMPRODUTO
            )
            GROUP BY CDFILIAL, NMFILIAL, CDEVENTO, NMEVENTO, CDLOJA, NMLOJA, CDCAIXA, NMCAIXA, CDPRODUTO, CDARVPROD, 
                    NMPRODUTO, VRUNITVEND, CDTIPOCUSTO, NMTIPOCUSTO, IDCUSFOPRECO, CDPRODUTOAVO, NMPRODUTOAVO
        ORDER BY CDFILIAL ASC, CDLOJA ASC, VRRENTA DESC
    ";
    
}